@@ -1,4 +1,5 @@
 import logging
+import os
 from typing import Union, Optional, Dict, List, Literal
 
 import pandas as pd
@@ -15,19 +16,23 @@ from .parsers import parse_prices, parse_loads, parse_generation, \
     parse_installed_capacity_per_plant, parse_crossborder_flows, \
     parse_unavailabilities, parse_contracted_reserve, parse_imbalance_prices_zip, \
     parse_imbalance_volumes_zip, parse_netpositions, parse_procured_balancing_capacity, \
-    parse_water_hydro,parse_aggregated_bids, parse_activated_balancing_energy_prices
+    parse_water_hydro,parse_aggregated_bids, parse_activated_balancing_energy_prices, \
+    parse_offshore_unavailability, parse_imbalance_volumes
 from .decorators import retry, paginated, year_limited, day_limited, documents_limited
 import warnings
 
 logger = logging.getLogger(__name__)
+warnings.filterwarnings('always')
 warnings.filterwarnings('ignore', category=XMLParsedAsHTMLWarning)
 
 __title__ = "entsoe-py"
-__version__ = "0.6.10"
+__version__ = "0.7.3"
 __author__ = "EnergieID.be, Frank Boerman"
 __license__ = "MIT"
 
-URL = 'https://web-api.tp.entsoe.eu/api'
+URL = os.getenv("ENTSOE_ENDPOINT_URL") or "https://web-api.tp.entsoe.eu/api"
+
+QUARTER_MTU_SDAC_GOLIVE =  pd.Timestamp('2025-10-01', tz='europe/amsterdam')
 
 
 class EntsoeRawClient:
@@ -41,8 +46,8 @@ class EntsoeRawClient:
         """
 
     def __init__(
-            self, api_key: str, session: Optional[requests.Session] = None,
-            retry_count: int = 1, retry_delay: int = 0,
+            self, api_key: str = None, session: Optional[requests.Session] = None,
+            retry_count: int = 3, retry_delay: int = 10,
             proxies: Optional[Dict] = None, timeout: Optional[int] = None):
         """
         Parameters
@@ -57,12 +62,17 @@ class EntsoeRawClient:
             requests proxies
         timeout : int
         """
-        if api_key is None:
-            raise TypeError("API key cannot be None")
         self.api_key = api_key
+        if self.api_key is None:
+            self.api_key = os.getenv("ENTSOE_API_KEY")
+        if self.api_key is None:
+            raise TypeError("API key cannot be None")
         if session is None:
             session = requests.Session()
         self.session = session
+        self.session.headers.update({
+            'user-agent': f'entsoe-py {__version__} (github.com/EnergieID/entsoe-py)'
+        })
         self.proxies = proxies
         self.retry_count = retry_count
         self.retry_delay = retry_delay
@@ -156,7 +166,8 @@ class EntsoeRawClient:
         return ret_str
 
     def query_day_ahead_prices(self, country_code: Union[Area, str],
-                               start: pd.Timestamp, end: pd.Timestamp) -> str:
+                               start: pd.Timestamp, end: pd.Timestamp,
+                               offset: int = 0, sequence: int = None) -> str:
         """
         Parameters
         ----------
@@ -172,8 +183,11 @@ class EntsoeRawClient:
         params = {
             'documentType': 'A44',
             'in_Domain': area.code,
-            'out_Domain': area.code
+            'out_Domain': area.code,
+            'offset': offset
         }
+        if sequence is not None:
+            params['classificationSequence_AttributeInstanceComponent.position'] = sequence
         response = self._base_request(params=params, start=start, end=end)
         return response.text
 
@@ -193,8 +207,8 @@ class EntsoeRawClient:
         -------
         str
         """
-        if process_type not in ['A51', 'A47']:
-            raise ValueError('processType allowed values: A51, A47')
+        if process_type not in ['A51', 'A52', 'A47']:
+            raise ValueError('processType allowed values: A51, A52, A47')
         area = lookup_area(country_code)
         params = {
             'documentType': 'A24',
@@ -371,7 +385,8 @@ class EntsoeRawClient:
 
     def query_generation_per_plant(
             self, country_code: Union[Area, str], start: pd.Timestamp,
-            end: pd.Timestamp, psr_type: Optional[str] = None, **kwargs) -> str:
+            end: pd.Timestamp, psr_type: Optional[str] = None,
+            eic_code: Optional[str] = None, **kwargs) -> str:
         """
         Parameters
         ----------
@@ -380,6 +395,8 @@ class EntsoeRawClient:
         end : pd.Timestamp
         psr_type : str
             filter on a single psr type
+        eic_code : str
+            filter on a single Generation Unit using its EIC Code
 
         Returns
         -------
@@ -393,6 +410,8 @@ class EntsoeRawClient:
         }
         if psr_type:
             params.update({'psrType': psr_type})
+        if eic_code:
+            params.update({'registeredResource': eic_code})
         response = self._base_request(params=params, start=start, end=end)
         return response.text
 
@@ -808,6 +827,29 @@ class EntsoeRawClient:
         response = self._base_request(params=params, start=start, end=end)
         return response.content
 
+    def query_current_balancing_state(
+            self, country_code: Union[Area, str], start: pd.Timestamp,
+            end: pd.Timestamp) -> str:
+        """
+        Parameters
+        ----------
+        country_code : Area|str
+        start : pd.Timestamp
+        end : pd.Timestamp
+
+        Returns
+        -------
+        str
+        """
+        area = lookup_area(country_code)
+        params = {
+            'documentType': 'A86',
+            'businessType': 'B33',
+            'area_Domain': area.code,
+        }
+        response = self._base_request(params=params, start=start, end=end)
+        return response.text
+
     def query_procured_balancing_capacity(
             self, country_code: Union[Area, str], start: pd.Timestamp,
             end: pd.Timestamp, process_type: str,
@@ -827,8 +869,8 @@ class EntsoeRawClient:
         -------
         bytes
         """
-        if process_type not in ['A51', 'A47']:
-            raise ValueError('processType allowed values: A51, A47')
+        if process_type not in ['A51', 'A52', 'A47']:
+            raise ValueError('processType allowed values: A51, A52, A47')
 
         area = lookup_area(country_code)
         params = {
@@ -1055,6 +1097,11 @@ class EntsoeRawClient:
             periodendupdate=periodendupdate, mRID=mRID, offset=offset)
         return content
 
+    def query_unavailability_of_offshore_grid(self, area_code: Union[Area, str], start: pd.Timestamp, end: pd.Timestamp):
+        return self._query_unavailability(
+            country_code=area_code, start=start, end=end, doctype='A79'
+        )
+
     def query_unavailability_of_production_units(
             self, country_code: Union[Area, str], start: pd.Timestamp,
             end: pd.Timestamp, docstatus: Optional[str] = None,
@@ -1152,9 +1199,10 @@ class EntsoeRawClient:
 class EntsoePandasClient(EntsoeRawClient):
     @year_limited
     def query_net_position(self, country_code: Union[Area, str],
-                            start: pd.Timestamp, end: pd.Timestamp, dayahead: bool = True, resolution: Literal['60min', '30min', '15min'] = '60min') -> pd.Series:
+                            start: pd.Timestamp, end: pd.Timestamp, dayahead: bool = True,
+                           resolution = None) -> pd.Series:
         """
-
+        forced to correct resolution for day ahead values
         Parameters
         ----------
         country_code
@@ -1165,18 +1213,42 @@ class EntsoePandasClient(EntsoeRawClient):
         -------
 
         """
+        if resolution is not None:
+            warnings.warn('The resolution parameter is deprecated and will be removed. This function will force the right resolution', DeprecationWarning)
         area = lookup_area(country_code)
         text = super(EntsoePandasClient, self).query_net_position(
             country_code=area, start=start, end=end, dayahead=dayahead)
-        series = parse_netpositions(text, resolution=resolution)
-        series = series.tz_convert(area.tz)
+        series = parse_netpositions(text)
+        if len(series) == 0:
+            raise NoMatchingDataError
+        if dayahead:
+            # This function should only return SDAC net positions for day ahead, which have a fixed defined resolution
+            # before 2025-10-01 its 60min, after 15min
+            # this is aligned on businessday in timezone europe/amsterdam
+            # some zones already publish in different resolution.
+            # for secondary auctions published on entsoe, use the query_day_ahead_prices_local function
+            if series.index.max() < QUARTER_MTU_SDAC_GOLIVE:
+                series = series.resample('h').first()
+            else:
+                series_60min = series[series.index < QUARTER_MTU_SDAC_GOLIVE]
+                series_15min = series[series.index >= QUARTER_MTU_SDAC_GOLIVE]
+
+                series = pd.concat([
+                    series_60min.resample('h').first(),
+                    series_15min
+                ]).sort_index()
+
+        series = series.tz_convert(area.tz).sort_index()
         series = series.truncate(before=start, after=end)
+        # because of the above fix we need to check again if any valid data exists after truncating
+        if len(series) == 0:
+            raise NoMatchingDataError
         return series
 
     @year_limited
     def query_aggregated_bids(self, country_code: Union[Area, str],
                               process_type: str,
-                            start: pd.Timestamp, end: pd.Timestamp) -> pd.Series:
+                              start: pd.Timestamp, end: pd.Timestamp) -> pd.DataFrame:
         """
 
         Parameters
@@ -1197,18 +1269,17 @@ class EntsoePandasClient(EntsoeRawClient):
         df = df.tz_convert(area.tz)
         df = df.truncate(before=start, after=end)
         return df
-    
-    @year_limited
+
+    # we need to do offset, but we also want to pad the days so wrap it in an internal call
     def query_day_ahead_prices(
             self, country_code: Union[Area, str],
             start: pd.Timestamp,
             end: pd.Timestamp,
-            resolution: Literal['60min', '30min', '15min'] = '60min') -> pd.Series:
+            resolution = None) -> pd.Series:
         """
         Parameters
         ----------
-        resolution: either 60min for hourly values,
-            30min for half-hourly values or 15min for quarterly values, throws error if type is not available
+        this will return the SDAC prices, forced to the correct resolution
         country_code : Area|str
         start : pd.Timestamp
         end : pd.Timestamp
@@ -1217,25 +1288,122 @@ class EntsoePandasClient(EntsoeRawClient):
         -------
         pd.Series
         """
-        if resolution not in ['60min', '30min', '15min']:
-            raise InvalidParameterError('Please choose either 60min, 30min or 15min')
+        if resolution is not None:
+            warnings.warn('The resolution parameter is deprecated and will be removed. This function will force the right SDAC resolution', DeprecationWarning)
         area = lookup_area(country_code)
         # we do here extra days at start and end to fix issue 187
-        text = super(EntsoePandasClient, self).query_day_ahead_prices(
-            country_code=area,
+        series = self._query_day_ahead_prices(
+            area,
             start=start-pd.Timedelta(days=1),
             end=end+pd.Timedelta(days=1)
         )
-        series = parse_prices(text)[resolution]
-        if len(series) == 0:
-            raise NoMatchingDataError
-        series = series.tz_convert(area.tz)
+        series = series.tz_convert(area.tz).sort_index()
         series = series.truncate(before=start, after=end)
         # because of the above fix we need to check again if any valid data exists after truncating
         if len(series) == 0:
             raise NoMatchingDataError
         return series
 
+    @year_limited
+    @documents_limited(100)
+    def _query_day_ahead_prices(
+            self, area: Area,
+            start: pd.Timestamp,
+            end: pd.Timestamp,
+            offset: int = 0) -> pd.Series:
+        text = super(EntsoePandasClient, self).query_day_ahead_prices(
+            area,
+            start=start,
+            end=end,
+            offset=offset,
+            sequence=1 if area.name in ['DE_LU', 'AT'] else None
+        )
+        series_all = parse_prices(text)
+
+        # This function should only return SDAC prices, which have a fixed defined resolution
+        # before 2025-10-01 its 60min, after 15min
+        # this is aligned on businessday in timezone europe/amsterdam
+        # some zones already publish in different resolution.
+        # for secondary auctions published on entsoe, use the query_day_ahead_prices_local function
+
+        series = pd.concat([x for x in series_all.values() if len(x) > 0]).sort_index().tz_convert('europe/amsterdam')
+        if len(series) == 0:
+            raise NoMatchingDataError
+        if series.index.max() < QUARTER_MTU_SDAC_GOLIVE:
+            series = series.resample('h').first()
+        else:
+            series_60min = series[series.index < QUARTER_MTU_SDAC_GOLIVE]
+            series_15min = series[series.index >= QUARTER_MTU_SDAC_GOLIVE]
+
+            series = pd.concat([
+                series_60min.resample('h').first(),
+                series_15min
+            ]).sort_index()
+
+        series = series.tz_convert(area.tz).sort_index()
+        series = series.truncate(before=start, after=end)
+        # because of the above fix we need to check again if any valid data exists after truncating
+        if len(series) == 0:
+            raise NoMatchingDataError
+        return series
+
+    # we need to do offset, but we also want to pad the days so wrap it in an internal call
+    def query_day_ahead_prices_local(
+            self, country_code: Union[Area, str],
+            sequence: int,
+            start: pd.Timestamp,
+            end: pd.Timestamp,
+            resolution: Literal['60min', '30min', '15min'] = '60min') -> pd.Series:
+        """
+        Parameters
+        ----------
+        this will return local auction prices that are published on entsoe, simply specify the sequence. sequence 1 is always SDAC
+        country_code : Area|str
+        start : pd.Timestamp
+        end : pd.Timestamp
+
+        Returns
+        -------
+        pd.Series
+        """
+        area = lookup_area(country_code)
+        # we do here extra days at start and end to fix issue 187
+        series = self._query_day_ahead_prices_local(
+            area,
+            sequence,
+            start=start-pd.Timedelta(days=1),
+            end=end+pd.Timedelta(days=1),
+            resolution=resolution
+        )
+        series = series.tz_convert(area.tz).sort_index()
+        series = series.truncate(before=start, after=end)
+        # because of the above fix we need to check again if any valid data exists after truncating
+        if len(series) == 0:
+            raise NoMatchingDataError
+        return series
+
+    @year_limited
+    @documents_limited(100)
+    def _query_day_ahead_prices_local(
+            self, area: Area,
+            sequence: int,
+            start: pd.Timestamp,
+            end: pd.Timestamp,
+            offset: int = 0,
+            resolution: Literal['60min', '30min', '15min'] = '60min') -> pd.Series:
+        text = super(EntsoePandasClient, self).query_day_ahead_prices(
+            area,
+            start=start,
+            end=end,
+            offset=offset,
+            sequence=sequence
+        )
+        series_all = parse_prices(text)
+        series = series_all[resolution]
+        if len(series) == 0:
+            raise NoMatchingDataError
+        return series
+
     @year_limited
     def query_load(self, country_code: Union[Area, str], start: pd.Timestamp,
                    end: pd.Timestamp) -> pd.DataFrame:
@@ -1329,6 +1497,8 @@ class EntsoePandasClient(EntsoeRawClient):
         text = super(EntsoePandasClient, self).query_generation_forecast(
             country_code=area, start=start, end=end, process_type=process_type)
         df = parse_generation(text, nett=nett)
+        if isinstance(df, pd.DataFrame):
+            df = df.rename(columns=lambda c: c.replace('Actual', 'Scheduled'))
         df = df.tz_convert(area.tz)
         df = df.truncate(before=start, after=end)
         return df
@@ -1780,7 +1950,7 @@ class EntsoePandasClient(EntsoeRawClient):
     @year_limited
     def query_imbalance_prices(
             self, country_code: Union[Area, str], start: pd.Timestamp,
-            end: pd.Timestamp, psr_type: Optional[str] = None) -> pd.DataFrame:
+            end: pd.Timestamp, psr_type: Optional[str] = None, include_resolution: bool = False) -> pd.DataFrame:
         """
         Parameters
         ----------
@@ -1789,7 +1959,8 @@ class EntsoePandasClient(EntsoeRawClient):
         end : pd.Timestamp
         psr_type : str
             filter query for a specific psr type
-
+        include_resolution: bool
+            Add resolution columns to the result
         Returns
         -------
         pd.DataFrame
@@ -1797,15 +1968,19 @@ class EntsoePandasClient(EntsoeRawClient):
         area = lookup_area(country_code)
         archive = super(EntsoePandasClient, self).query_imbalance_prices(
             country_code=area, start=start, end=end, psr_type=psr_type)
-        df = parse_imbalance_prices_zip(zip_contents=archive)
+        df = parse_imbalance_prices_zip(zip_contents=archive, include_resolution=include_resolution)
         df = df.tz_convert(area.tz)
         df = df.truncate(before=start, after=end)
+        # 
+        if include_resolution:
+            df = df.rename(columns={'Resolution Long': 'Resolution'})
+            df.drop(columns=['Resolution Short'], inplace=True)
         return df
 
     @year_limited
     def query_imbalance_volumes(
             self, country_code: Union[Area, str], start: pd.Timestamp,
-            end: pd.Timestamp, psr_type: Optional[str] = None) -> pd.DataFrame:
+            end: pd.Timestamp, psr_type: Optional[str] = None, include_resolution=False) -> pd.DataFrame:
         """
         Parameters
         ----------
@@ -1814,7 +1989,8 @@ class EntsoePandasClient(EntsoeRawClient):
         end : pd.Timestamp
         psr_type : str
             filter query for a specific psr type
-
+        include_resolution: bool
+            include resolution column in the result
         Returns
         -------
         pd.DataFrame
@@ -1822,7 +1998,29 @@ class EntsoePandasClient(EntsoeRawClient):
         area = lookup_area(country_code)
         archive = super(EntsoePandasClient, self).query_imbalance_volumes(
             country_code=area, start=start, end=end, psr_type=psr_type)
-        df = parse_imbalance_volumes_zip(zip_contents=archive)
+        df = parse_imbalance_volumes_zip(zip_contents=archive, include_resolution=include_resolution)
+        df = df.tz_convert(area.tz)
+        df = df.truncate(before=start, after=end)
+        return df
+
+    @year_limited
+    def query_current_balancing_state(
+            self, country_code: Union[Area, str], start: pd.Timestamp,
+            end: pd.Timestamp) -> pd.DataFrame:
+        """
+        Parameters
+        ----------
+        country_code : Area|str
+        start : pd.Timestamp
+        end : pd.Timestamp
+        Returns
+        -------
+        pd.DataFrame
+        """
+        area = lookup_area(country_code)
+        text = super(EntsoePandasClient, self).query_current_balancing_state(
+            country_code=area, start=start, end=end)
+        df = -1*parse_imbalance_volumes(text)
         df = df.tz_convert(area.tz)
         df = df.truncate(before=start, after=end)
         return df
@@ -2045,6 +2243,16 @@ class EntsoePandasClient(EntsoeRawClient):
         df = df[(df['start'] < end) | (df['end'] > start)]
         return df
 
+    def query_unavailability_of_offshore_grid(self, area_code: Union[Area, str],
+                                              start: pd.Timestamp, end: pd.Timestamp
+                                              ) -> pd.DataFrame:
+        zipfile = super(EntsoePandasClient, self)._query_unavailability(
+            country_code=area_code, start=start, end=end, doctype='A79'
+        )
+        df = parse_offshore_unavailability(zipfile)
+        return df
+
+
     def query_unavailability_of_generation_units(
             self, country_code: Union[Area, str], start: pd.Timestamp,
             end: pd.Timestamp, docstatus: Optional[str] = None,
@@ -2159,7 +2367,8 @@ class EntsoePandasClient(EntsoeRawClient):
             self, country_code: Union[Area, str], start: pd.Timestamp,
             end: pd.Timestamp, psr_type: Optional[str] = None,
             include_eic: bool = False,
-             **kwargs) -> pd.DataFrame:
+            eic_code: Optional[str] = None,
+            **kwargs) -> pd.DataFrame:
         """
         Parameters
         ----------
@@ -2170,6 +2379,8 @@ class EntsoePandasClient(EntsoeRawClient):
             filter on a single psr type
         include_eic: bool
             if True also include the eic code in the output
+        eic_code : str
+            filter on a single Generation Unit using its EIC Code
 
         Returns
         -------
@@ -2177,7 +2388,9 @@ class EntsoePandasClient(EntsoeRawClient):
         """
         area = lookup_area(country_code)
         text = super(EntsoePandasClient, self).query_generation_per_plant(
-            country_code=area, start=start, end=end, psr_type=psr_type)
+            country_code=area, start=start, end=end, psr_type=psr_type,
+            eic_code=eic_code,
+        )
         df = parse_generation(text, per_plant=True, include_eic=include_eic)
         df.columns = df.columns.set_levels(df.columns.levels[0].str.encode('latin-1').str.decode('utf-8'), level=0)
         df = df.tz_convert(area.tz)
