@@ -183,110 +183,119 @@ PSRTYPE_MAPPINGS = {
     'B22': 'DC Link',
     'B23': 'Substation',
     'B24': 'Transformer',
-    'B25': 'Energy storage'}
+    'B25': 'Energy storage'
+}
 
 # https://transparency.entsoe.eu/content/static_content/Static%20content/web%20api/Guide.html#_docstatus
-DOCSTATUS = {'A01': 'Intermediate',
-             'A02': 'Final',
-             'A05': 'Active',
-             'A09': 'Cancelled',
-             'A13': 'Withdrawn',
-             'X01': 'Estimated'}
+DOCSTATUS = {
+    'A01': 'Intermediate',
+    'A02': 'Final',
+    'A05': 'Active',
+    'A09': 'Cancelled',
+    'A13': 'Withdrawn',
+    'X01': 'Estimated'
+}
 
 # https://transparency.entsoe.eu/content/static_content/Static%20content/web%20api/Guide.html#_businesstype
-BSNTYPE = { 'A01': 'Production',
-           'A04': 'Consumption',
-           'A14': 'Aggregated energy data',
-           'A19': 'Balance energy deviation',
-           'A25': 'General Capacity Information',
-           'A29': 'Already allocated capacity (AAC)',
-           'A43': 'Requested capacity (without price)',
-           'A46': 'System Operator redispatching',
-           'A53': 'Planned maintenance',
-           'A54': 'Unplanned outage',
-           'A60': 'Minimum possible',
-           'A61': 'Maximum possible',
-           'A85': 'Internal redispatch',
-           'A91': 'Positive forecast margin (if installed capacity > load forecast)',
-           'A92': 'Negative forecast margin (if load forecast > installed capacity)',
-           'A93': 'Wind generation',
-           'A94': 'Solar generation',
-           'A95': 'Frequency containment reserve',
-           'A96': 'Automatic frequency restoration reserve',
-           'A97': 'Manual frequency restoration reserve',
-           'A98': 'Replacement reserve',
-           'B01': 'Interconnector network evolution',
-           'B02': 'Interconnector network dismantling',
-           'B03': 'Counter trade',
-           'B04': 'Congestion costs',
-           'B05': 'Capacity allocated (including price)',
-           'B07': 'Auction revenue',
-           'B08': 'Total nominated capacity',
-           'B09': 'Net position',
-           'B10': 'Congestion income',
-           'B11': 'Production unit',
-           'B33': 'Area Control Error',
-           'B74': 'Offer',
-           'B75': 'Need', 
-           'B95': 'Procured capacity',
-           'C22': 'Shared Balancing Reserve Capacity', 
-           'C23': 'Share of reserve capacity',
-           'C24': 'Actual reserve capacity'}
+BSNTYPE = { 
+    'A01': 'Production',
+    'A04': 'Consumption',
+    'A14': 'Aggregated energy data',
+    'A19': 'Balance energy deviation',
+    'A25': 'General Capacity Information',
+    'A29': 'Already allocated capacity (AAC)',
+    'A43': 'Requested capacity (without price)',
+    'A46': 'System Operator redispatching',
+    'A53': 'Planned maintenance',
+    'A54': 'Unplanned outage',
+    'A60': 'Minimum possible',
+    'A61': 'Maximum possible',
+    'A85': 'Internal redispatch',
+    'A91': 'Positive forecast margin (if installed capacity > load forecast)',
+    'A92': 'Negative forecast margin (if load forecast > installed capacity)',
+    'A93': 'Wind generation',
+    'A94': 'Solar generation',
+    'A95': 'Frequency containment reserve',
+    'A96': 'Automatic frequency restoration reserve',
+    'A97': 'Manual frequency restoration reserve',
+    'A98': 'Replacement reserve',
+    'B01': 'Interconnector network evolution',
+    'B02': 'Interconnector network dismantling',
+    'B03': 'Counter trade',
+    'B04': 'Congestion costs',
+    'B05': 'Capacity allocated (including price)',
+    'B07': 'Auction revenue',
+    'B08': 'Total nominated capacity',
+    'B09': 'Net position',
+    'B10': 'Congestion income',
+    'B11': 'Production unit',
+    'B33': 'Area Control Error',
+    'B74': 'Offer',
+    'B75': 'Need', 
+    'B95': 'Procured capacity',
+    'C22': 'Shared Balancing Reserve Capacity', 
+    'C23': 'Share of reserve capacity',
+    'C24': 'Actual reserve capacity'
+}
 
 # https://transparency.entsoe.eu/content/static_content/Static%20content/web%20api/Guide.html#_contract_marketagreement_type_type_marketagreement_type
-MARKETAGREEMENTTYPE = {'A01': 'Daily',
-                       'A02': 'Weekly',
-                       'A03': 'Monthly',
-                       'A04': 'Yearly',
-                       'A05': 'Total',
-                       'A06': 'Long term',
-                       'A07': 'Intraday',
-                       'A13': 'Hourly'}
+MARKETAGREEMENTTYPE = {
+    'A01': 'Daily',
+    'A02': 'Weekly',
+    'A03': 'Monthly',
+    'A04': 'Yearly',
+    'A05': 'Total',
+    'A06': 'Long term',
+    'A07': 'Intraday',
+    'A13': 'Hourly'
+}
 
 # https://transparency.entsoe.eu/content/static_content/Static%20content/web%20api/Guide.html#_documenttype
-DOCUMENTTYPE = {'A09': 'Finalised schedule',
-                'A11': 'Aggregated energy data report',
-                'A15': 'Acquiring system operator reserve schedule',
-                'A24': 'Bid document',
-                'A25': 'Allocation result document',
-                'A26': 'Capacity document',
-                'A31': 'Agreed capacity',
-                'A38': 'Reserve allocation result document',
-                'A44': 'Price Document',
-                'A61': 'Estimated Net Transfer Capacity',
-                'A63': 'Redispatch notice',
-                'A65': 'System total load',
-                'A68': 'Installed generation per type',
-                'A69': 'Wind and solar forecast',
-                'A70': 'Load forecast margin',
-                'A71': 'Generation forecast',
-                'A72': 'Reservoir filling information',
-                'A73': 'Actual generation',
-                'A74': 'Wind and solar generation',
-                'A75': 'Actual generation per type',
-                'A76': 'Load unavailability',
-                'A77': 'Production unavailability',
-                'A78': 'Transmission unavailability',
-                'A79': 'Offshore grid infrastructure unavailability',
-                'A80': 'Generation unavailability',
-                'A81': 'Contracted reserves',
-                'A82': 'Accepted offers',
-                'A83': 'Activated balancing quantities',
-                'A84': 'Activated balancing prices',
-                'A85': 'Imbalance prices',
-                'A86': 'Imbalance volume',
-                'A87': 'Financial situation',
-                'A88': 'Cross border balancing',
-                'A89': 'Contracted reserve prices',
-                'A90': 'Interconnection network expansion',
-                'A91': 'Counter trade notice',
-                'A92': 'Congestion costs',
-                'A93': 'DC link capacity',
-                'A94': 'Non EU allocations',
-                'A95': 'Configuration document',
-                'B11': 'Flow-based allocations',
-                'B17': 'Aggregated netted external TSO schedule document',
-                'B45': 'Bid Availability Document'}
+DOCUMENTTYPE = {
+    'A09': 'Finalised schedule',
+    'A11': 'Aggregated energy data report',
+    'A15': 'Acquiring system operator reserve schedule',
+    'A24': 'Bid document',
+    'A25': 'Allocation result document',
+    'A26': 'Capacity document',
+    'A31': 'Agreed capacity',
+    'A38': 'Reserve allocation result document',
+    'A44': 'Price Document',
+    'A61': 'Estimated Net Transfer Capacity',
+    'A63': 'Redispatch notice',
+    'A65': 'System total load',
+    'A68': 'Installed generation per type',
+    'A69': 'Wind and solar forecast',
+    'A70': 'Load forecast margin',
+    'A71': 'Generation forecast',
+    'A72': 'Reservoir filling information',
+    'A73': 'Actual generation',
+    'A74': 'Wind and solar generation',
+    'A75': 'Actual generation per type',
+    'A76': 'Load unavailability',
+    'A77': 'Production unavailability',
+    'A78': 'Transmission unavailability',
+    'A79': 'Offshore grid infrastructure unavailability',
+    'A80': 'Generation unavailability',
+    'A81': 'Contracted reserves',
+    'A82': 'Accepted offers',
+    'A83': 'Activated balancing quantities',
+    'A84': 'Activated balancing prices',
+    'A85': 'Imbalance prices',
+    'A86': 'Imbalance volume',
+    'A87': 'Financial situation',
+    'A88': 'Cross border balancing',
+    'A89': 'Contracted reserve prices',
+    'A90': 'Interconnection network expansion',
+    'A91': 'Counter trade notice',
+    'A92': 'Congestion costs',
+    'A93': 'DC link capacity',
+    'A94': 'Non EU allocations',
+    'A95': 'Configuration document',
+    'B11': 'Flow-based allocations',
+    'B17': 'Aggregated netted external TSO schedule document',
+    'B45': 'Bid Availability Document'
+}
 
 # https://transparency.entsoe.eu/content/static_content/Static%20content/web%20api/Guide.html#_processtype
 PROCESSTYPE = {
@@ -324,11 +333,11 @@ NEIGHBOURS = {
     'HU': ['AT', 'HR', 'RO', 'RS', 'SI', 'SK', 'UA'],
     'IT_NORD': ['CH', 'DE_AT_LU', 'FR', 'SI', 'AT', 'IT_CNOR'],
     'ES': ['FR', 'PT'],
-    'SI': ['AT', 'DE_AT_LU', 'HR', 'IT_NORD'],
+    'SI': ['AT', 'DE_AT_LU', 'HR', 'IT_NORD', 'HU'],
     'RS': ['AL', 'BA', 'BG', 'HR', 'HU', 'ME', 'MK', 'RO'],
     'PL': ['CZ', 'DE_AT_LU', 'DE_LU', 'LT', 'SE_4', 'SK', 'UA'],
     'ME': ['AL', 'BA', 'RS'],
-    'DK_1': ['DE_AT_LU', 'DE_LU', 'DK_2', 'NO_2', 'SE_3', 'NL'],
+    'DK_1': ['DE_AT_LU', 'DE_LU', 'DK_2', 'NO_2', 'SE_3', 'NL', 'GB'],
     'RO': ['BG', 'HU', 'RS', 'UA'],
     'LT': ['BY', 'LV', 'PL', 'RU_KGD', 'SE_4'],
     'BG': ['GR', 'MK', 'RO', 'RS', 'TR'],
