@@ -45,7 +45,7 @@ def parse_prices(xml_text):
     return series
 
 
-def parse_netpositions(xml_text, resolution):
+def parse_netpositions(xml_text):
     """
 
     Parameters
@@ -58,7 +58,7 @@ def parse_netpositions(xml_text, resolution):
     """
     series_all = []
     for soup in _extract_timeseries(xml_text):
-        series = _parse_timeseries_generic(soup)[resolution]
+        series = _parse_timeseries_generic(soup, merge_series=True)
         if series is None:
             continue
         if 'REGION' in soup.find('out_domain.mrid').text:
@@ -69,7 +69,8 @@ def parse_netpositions(xml_text, resolution):
         # take the absolute value and correct for region
         #TODO: possible change this or remove this warning after helpdesk got back to me
         series_all.append(factor*series.abs())
-
+    if len(series_all) == 0:
+        return pd.Series()
     series_all = pd.concat(series_all).sort_index()
     return series_all
 
@@ -252,12 +253,8 @@ def parse_crossborder_flows(xml_text):
     -------
     pd.Series
     """
-    series = []
-    for soup in _extract_timeseries(xml_text):
-        series.append(_parse_crossborder_flows_timeseries(soup))
-    series = pd.concat(series)
-    series = series.sort_index()
-    return series
+    return _parse_timeseries_generic_whole(xml_text, to_float=True)
+
     
 def parse_activated_balancing_energy_prices(xml_text):
     """
@@ -277,18 +274,18 @@ def parse_activated_balancing_energy_prices(xml_text):
     df.sort_index(inplace=True)
     return df
 
-def parse_imbalance_prices(xml_text):
+def parse_imbalance_prices(xml_text, include_resolution=False):
     """
     Parameters
     ----------
     xml_text : str
-
+    include_resolution: bool
     Returns
     -------
     pd.DataFrame
     """
     timeseries_blocks = _extract_timeseries(xml_text)
-    frames = (_parse_imbalance_prices_timeseries(soup)
+    frames = (_parse_imbalance_prices_timeseries(soup, include_resolution)
               for soup in timeseries_blocks)
     df = pd.concat(frames, axis=1)
     df = df.stack().unstack()  # ad-hoc fix to prevent column splitting by NaNs
@@ -296,18 +293,18 @@ def parse_imbalance_prices(xml_text):
     return df
 
 
-def parse_imbalance_volumes(xml_text):
+def parse_imbalance_volumes(xml_text, include_resolution=False):
     """
     Parameters
     ----------
     xml_text : str
-
+    include_resolution: bool
     Returns
     -------
     pd.DataFrame
     """
     timeseries_blocks = _extract_timeseries(xml_text)
-    frames = (_parse_imbalance_volumes_timeseries(soup)
+    frames = (_parse_imbalance_volumes_timeseries(soup, include_resolution)
               for soup in timeseries_blocks)
     df = pd.concat(frames, axis=1)
     df = df.stack().unstack()  # ad-hoc fix to prevent column splitting by NaNs
@@ -499,12 +496,12 @@ def _parse_contracted_reserve_series(soup, tz, label):
     df.columns = pd.MultiIndex.from_product([df.columns, [direction]])
     return df
 
-def parse_imbalance_prices_zip(zip_contents: bytes) -> pd.DataFrame:
+def parse_imbalance_prices_zip(zip_contents: bytes, include_resolution: bool =False) -> pd.DataFrame:
     """
     Parameters
     ----------
     zip_contents : bytes
-
+    include_resolution: bool
     Returns
     -------
     pd.DataFrame
@@ -513,7 +510,7 @@ def parse_imbalance_prices_zip(zip_contents: bytes) -> pd.DataFrame:
         with zipfile.ZipFile(BytesIO(archive), 'r') as arc:
             for f in arc.infolist():
                 if f.filename.endswith('xml'):
-                    frame = parse_imbalance_prices(xml_text=arc.read(f))
+                    frame = parse_imbalance_prices(xml_text=arc.read(f), include_resolution= include_resolution)
                     yield frame
 
     frames = gen_frames(zip_contents)
@@ -564,12 +561,12 @@ def _parse_activated_balancing_energy_prices_timeseries(soup) -> pd.DataFrame:
     df = df.infer_objects(copy=False).ffill()
     return df
 
-def _parse_imbalance_prices_timeseries(soup) -> pd.DataFrame:
+def _parse_imbalance_prices_timeseries(soup, include_resolution=False) -> pd.DataFrame:
     """
     Parameters
     ----------
     soup : bs4.element.tag
-
+    include_resolution: bool
     Returns
     -------
     pd.DataFrame
@@ -595,15 +592,27 @@ def _parse_imbalance_prices_timeseries(soup) -> pd.DataFrame:
     df.columns.name = None
     df.rename(columns={'A04': 'Long', 'A05': 'Short',
                        'None': 'Price for Consumption'}, inplace=True)
-
+    if include_resolution:
+        period = soup.find('period')
+        resolution = None
+        if period is not None and period.find('resolution') is not None:
+            resolution = period.find('resolution').text
+        if 'Long' in df.columns:
+            df['Resolution Long'] = resolution
+        else:
+            df['Resolution Long'] = None
+        if 'Short' in df.columns:
+            df['Resolution Short'] = resolution
+        else:
+            df['Resolution Short'] = None
     return df
 
-def parse_imbalance_volumes_zip(zip_contents: bytes) -> pd.DataFrame:
+def parse_imbalance_volumes_zip(zip_contents: bytes, include_resolution:bool = False) -> pd.DataFrame:
     """
     Parameters
     ----------
     zip_contents : bytes
-
+    include_resolution : bool
     Returns
     -------
     pd.DataFrame
@@ -612,7 +621,7 @@ def parse_imbalance_volumes_zip(zip_contents: bytes) -> pd.DataFrame:
         with zipfile.ZipFile(BytesIO(archive), 'r') as arc:
             for f in arc.infolist():
                 if f.filename.endswith('xml'):
-                    frame = parse_imbalance_volumes(xml_text=arc.read(f))
+                    frame = parse_imbalance_volumes(xml_text=arc.read(f), include_resolution=include_resolution)
                     yield frame
 
     frames = gen_frames(zip_contents)
@@ -620,12 +629,12 @@ def parse_imbalance_volumes_zip(zip_contents: bytes) -> pd.DataFrame:
     df.sort_index(inplace=True)
     return df
 
-def _parse_imbalance_volumes_timeseries(soup) -> pd.DataFrame:
+def _parse_imbalance_volumes_timeseries(soup, include_resolution) -> pd.DataFrame:
     """
     Parameters
     ----------
     soup : bs4.element.tag
-
+    include_resolution: bool
     Returns
     -------
     pd.DataFrame
@@ -636,7 +645,8 @@ def _parse_imbalance_volumes_timeseries(soup) -> pd.DataFrame:
         # time series uses flow direction codes
         flow_direction_factor = {
             'A01': 1, # in
-            'A02': -1 # out
+            'A02': -1, # out
+            'A03': 1,  # symmetric / in-balance
         }[flow_direction.text]
     else:
         # time series uses positive and negative values
@@ -651,10 +661,12 @@ def _parse_imbalance_volumes_timeseries(soup) -> pd.DataFrame:
         tx = pd.date_range(start=start, end=end, freq=resolution, inclusive='left')
         points = period.find_all('point')
 
-        for dt, point in zip(tx, points):
-            df.loc[dt, 'Imbalance Volume'] = \
+        for point in points:
+            position = int(point.find('position').text)
+            df.loc[tx[position-1], 'Imbalance Volume'] = \
                 float(point.find('quantity').text) * flow_direction_factor
-
+        if include_resolution:
+            df["Resolution"] = resolution
     df.set_index(['Imbalance Volume'])
 
     return df
@@ -941,15 +953,38 @@ def parse_unavailabilities(response: bytes, doctype: str) -> pd.DataFrame:
     return df
 
 
+def parse_offshore_unavailability(response: bytes) -> pd.DataFrame:
+    """
+    offshore has slightly different structure so use seperate parser. this also enables using the new generic parsers as well
+    """
+    df = {}
+    with zipfile.ZipFile(BytesIO(response), 'r') as arc:
+        for f in arc.infolist():
+            if f.filename.endswith('xml'):
+                for series in _extract_timeseries(arc.read(f)):
+                    asset = series.find('Asset_RegisteredResource'.lower())
+                    name = "|".join([asset.find(x).text for x in ['mrid', 'name', 'location.name']])
+                    df[name] = _parse_timeseries_generic(series, merge_series=True, period_name='windpowerfeedin_period')
+    return pd.DataFrame(df)
+
+
 def _available_period(timeseries: bs4.BeautifulSoup) -> list:
     # if not timeseries:
     #    return
     for period in timeseries.find_all('available_period'):
-        start, end = pd.Timestamp(period.timeinterval.start.text), pd.Timestamp(
+        start_p, end_p = pd.Timestamp(period.timeinterval.start.text), pd.Timestamp(
             period.timeinterval.end.text)
         res = period.resolution.text
-        pstn, qty = period.point.position.text, period.point.quantity.text
-        yield [start, end, res, pstn, qty]
+        pts = period.find_all('point')
+        for idx, pt in enumerate(pts):
+            pstn, qty = pt.position.text, pt.quantity.text
+            start = start_p + (pd.Timedelta(res) * (int(pstn) - 1))
+            if idx < (len(pts) - 1):
+                pstn_next = pts[idx + 1].position.text
+                end = start_p + (pd.Timedelta(res) * (int(pstn_next) - 1))
+            else:
+                end = end_p
+            yield [start, end, res, pstn, qty]
 
 
 def _outage_parser(xml_file: bytes, headers, ts_func) -> pd.DataFrame:
